import moment from 'moment';
import React, {Component} from 'react';
import {
  ActivityIndicator,
  Alert,
  Dimensions,
  ImageBackground,
  Platform,
  ScrollView,
  StatusBar,
  Text,
  TouchableOpacity,
  View,
  Image,
} from 'react-native';
import RNExitApp from 'react-native-exit-app';
import {KeyboardAwareScrollView} from 'react-native-keyboard-aware-scroll-view';
import MaterialIcons from 'react-native-vector-icons/MaterialIcons';
import {connect} from 'react-redux';
import * as colors from '../../assets/colors';
import img_popup from '../../assets/images/img_popup.png';
import ic_clock from '../../assets/images/ic_clock.png';
import {AUDIO_RECORDING} from '../../common/constant';
import SharePreference from '../../common/SharePreference';
import Config from '../../Config';
import {
  REQUEST_IS_FINISH,
  REQUEST_COMPONENTS_KEY_PAGES,
  REQUEST_UPLOAD_RECORD,
} from '../../redux/actions';
import {request} from '../../redux/api';
import CountDown from '../count-down';
import Button from './button';
import {styles} from './styles';
// Detect screen width and height
const {width, height} = Dimensions.get('window');
MaterialIcons.loadFont();

class OnboardingScreens extends Component {
  // Props for ScrollView component
  static defaultProps = {
    // Arrange screens horizontally
    horizontal: true,
    // Scroll exactly to the next screen, instead of continous scrolling
    pagingEnabled: true,
    // Hide all scroll indicators
    showsHorizontalScrollIndicator: false,
    showsVerticalScrollIndicator: false,
    // Do not bounce when the end is reached
    bounces: false,
    // Do not scroll to top when the status bar is tapped
    scrollsToTop: false,
    // Remove offscreen child views
    removeClippedSubviews: true,
    // Do not adjust content behind nav-, tab- or toolbars automatically
    automaticallyAdjustContentInsets: false,
    // Fisrt is screen is active
    index: 0,
  };

  state = this.initState(this.props);

  /**
   * Initialize the state
   */
  initState(props) {
    // Get the total number of slides passed as children
    const total = props.children ? props.children.length || 1 : 0,
      // Current index
      index = total > 1 ? Math.min(props.index, total - 1) : 0,
      // Current offset
      offset = width * index;

    const state = {
      total,
      index,
      offset,
      width,
      height,
      loading: false,
    };

    // Component internals as a class property,
    // and not state to avoid component re-renders when updated
    this.internals = {
      isScrolling: false,
      offset,
    };

    return state;
  }

  /**
   * Scroll begin handler
   * @param {object} e native event
   */
  onScrollBegin = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = true;
  };

  /**
   * Scroll end handler
   * @param {object} e native event
   */
  onScrollEnd = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = false;

    // Update index
    this.updateIndex(
      e.nativeEvent.contentOffset
        ? e.nativeEvent.contentOffset.x
        : // When scrolled with .scrollTo() on Android there is no contentOffset
          e.nativeEvent.position * this.state.width,
    );
  };

  /*
   * Drag end handler
   * @param {object} e native event
   */
  onScrollEndDrag = (e) => {
    const {
        contentOffset: {x: newOffset},
      } = e.nativeEvent,
      {children} = this.props,
      {index} = this.state,
      {offset} = this.internals;
    // Update internal isScrolling state
    // if swiped right on the last slide
    // or left on the first one
    if (
      offset === newOffset &&
      (index === 0 || index === children.length - 1)
    ) {
      this.internals.isScrolling = false;
    }
  };

  /**
   * Update index after scroll
   * @param {object} offset content offset
   */
  updateIndex = (offset) => {
    const {dispatch} = this.props;
    const state = this.state,
      diff = offset - this.internals.offset,
      step = state.width;
    let index = state.index;
    // Do nothing if offset didn't change
    if (!diff) {
      return;
    }

    // Make sure index is always an integer
    index = parseInt(index + Math.round(diff / step), 10);

    // Update internal offset
    this.internals.offset = offset;
    // Update index in the state
    this.setState({
      index,
    });
    dispatch({type: REQUEST_COMPONENTS_KEY_PAGES, payload: {keyPage: index}});
    // this.props.setMute();
    this.props.stopSound();
    this.dispatchAnswer();
  };

  dispatchAnswer = () => {
    const {dispatch, submitData} = this.props;

    const arrAnswer = submitData.user_answer;
    const recordExisted = arrAnswer.find(
      (answer) =>
        answer.type === AUDIO_RECORDING && answer.user_answer.value === '',
    );
    if (recordExisted) {
      // console.log(999, 'upload', recordExisted);
      //goi action upload file record
      dispatch({type: REQUEST_UPLOAD_RECORD, payload: recordExisted});
    }
  };

  /**
   * Swipe one slide forward
   */
  swipe = () => {
    // Ignore if already scrolling or if there is less than 2 slides
    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    const state = this.state,
      diff = this.state.index + 1,
      x = diff * state.width,
      y = 0;
    // Call scrollTo on scrollView component to perform the swipe
    this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

    // Update internal scroll state
    this.internals.isScrolling = true;

    // Trigger onScrollEnd manually on android
    if (Platform.OS === 'android') {
      setImmediate(() => {
        this.onScrollEnd({
          nativeEvent: {
            position: diff,
          },
        });
      });
    }
  };

  prev = () => {
    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    const state = this.state,
      diff = this.state.index - 1,
      x = diff * state.width,
      y = 0;
    // Call scrollTo on scrollView component to perform the swipe
    this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

    // Update internal scroll state
    this.internals.isScrolling = true;

    // Trigger onScrollEnd manually on android
    if (Platform.OS === 'android') {
      setImmediate(() => {
        this.onScrollEnd({
          nativeEvent: {
            position: diff,
          },
        });
      });
    }
  };
  /**
   * Render ScrollView component
   * @param {array} slides to swipe through
   */
  renderScrollView = (pages) => {
    return (
      <ScrollView
        ref={(component) => {
          this.scrollView = component;
        }}
        {...this.props}
        style={{flex: 1}}
        onScrollBeginDrag={this.onScrollBegin}
        onMomentumScrollEnd={this.onScrollEnd}
        onScrollEndDrag={this.onScrollEndDrag}>
        {pages.map((page, i) => (
          // Render each slide inside a View
          <KeyboardAwareScrollView key={i}>
            <View style={[styles.slide]}>{page}</View>
          </KeyboardAwareScrollView>
        ))}
      </ScrollView>
    );
  };

  /**
   * Render pagination indicators
   */
  renderPagination = () => {
    if (this.state.total <= 1) {
      return null;
    }

    const ActiveDot = <View style={[styles.dot, styles.activeDot]} />,
      Dot = <View style={styles.dot} />;

    let dots = [];

    for (let key = 0; key < this.state.total; key++) {
      dots.push(
        key === this.state.index
          ? // Active dot
            React.cloneElement(ActiveDot, {key})
          : // Other dots
            React.cloneElement(Dot, {key}),
      );
    }

    return (
      <View pointerEvents="none" style={[styles.pagination, styles.fullScreen]}>
        {dots}
      </View>
    );
  };
  renderQuestions = () => {
    const last = this.state.index + 1;
    const total = this.state.total;
    this.percentQuestion();
    return (
      <View style={[styles.question]}>
        <Text style={styles.questionText}>
          {last < 10 ? '0' + last : last}/{total}
        </Text>
        {/* <View style={styles.line} /> */}
      </View>
    );
  };
  /**
   * Render Continue or Done button
   */
  renderButton = () => {
    const lastScreen = this.state.index === this.state.total - 1;
    return (
      <View pointerEvents="box-none" style={[styles.buttonWrapper]}>
        {!lastScreen && (
          <Button
            text="Câu tiếp"
            bgColor={colors.color_blue_2}
            onPress={() => this.swipe()}
          />
        )}
      </View>
    );
  };

  confirmAnwer = (type) => {
    const {user_answer} = this.props.submitData;
    const {children, isFinishQiz} = this.props;
    if (isFinishQiz) {
      return;
    }
    if (type === 0) {
      this.dispatchAnswer();
      Alert.alert(
        'Thông báo',
        `Bạn đã trả lời ${user_answer.length}/${children.length} câu hỏi.\nHoàn thành bài kiểm tra?`,
        [{text: 'Huỷ bỏ'}, {text: 'Đồng ý', onPress: () => this.acceptAnwer()}],
      );
    } else {
      Alert.alert(
        'Thông báo',
        `Bạn đã hết thời gian làm bài.\nBạn đã hoàn thành ${user_answer.length}/${children.length} câu hỏi.`,
        [
          {
            text: 'Ok',
            style: 'cancel',
          },
          {cancelable: false},
        ],
      );
      setTimeout(() => {
        this.acceptAnwer();
      }, 2000);
    }
  };

  acceptAnwer = () => {
    const {data, submitData} = this.props;
    let dataReview = [];
    submitData.user_answer.forEach((item) => {
      const data = {
        question_configuration: item.question_configuration,
        question_id: item.question_id,
        section_id: item.section_id,
        section_tag: item.section_tag,
        user_answer: item.user_answer,
      };
      dataReview.push(data);
    });
    // dispath action request submit answer
    const dataAnswer = {
      user_test_id: data.data.id,
      data: JSON.stringify(dataReview),
      started_at: SharePreference.startTest,
      finished_at: moment().unix(),
    };
    console.log('345', dataAnswer);
    this.postSubmitAnswer(dataAnswer);
  };

  postSubmitAnswer = (params) => {
    const {children, dispatch, navigation} = this.props;
    const {user_answer} = this.props.submitData;

    dispatch({type: REQUEST_IS_FINISH, payload: {isFinish: true}});
    const numberAnswer = user_answer.length + '/' + children.length;
    navigation.navigate('Congratulations', {numberAnswer: numberAnswer});
    // this.setState({loading: true});
    // request({
    //   url: `${Config.url}/api/test/submit-answers`,
    //   method: 'post',
    //   headers: {Authorization: SharePreference.token_hotqa},
    //   body: params,
    // })
    //   .then((res) => {
    //     if (res.body.status === 200 && res.body.success) {
    // Alert.alert(
    //   '',
    //   'Chúc mừng bạn đã Kết thúc bài kiểm tra.\nBạn có thể xem đáp án của các câu hỏi tại đây.',
    //   [
    //     {
    //       text: 'Ok',
    //     },
    //   ],
    // );
    //     } else {
    //       this.dialogNotify(res.body.message, params);
    //     }
    //     this.setState({loading: false});
    //   })
    //   .catch((e) => {
    //     const error = JSON.parse(e.message);
    //     this.dialogNotify(error.message, params);
    //     this.setState({loading: false});
    //   });
  };

  dialogNotify = (message, params) => {
    const {route} = this.props;
    Alert.alert('Lỗi', message, [
      {text: 'Huỷ bỏ', onPress: () => (route ? route.goBack() : {})},
      {text: 'Thử lại', onPress: () => this.postSubmitAnswer(params)},
    ]);
  };

  renderButtonPrev = () => {
    const lastScreen = this.state.index >= 1;
    return (
      <View pointerEvents="box-none" style={[styles.buttonPrev]}>
        {lastScreen && (
          <Button
            text="Quay lại"
            bgColor={colors.color_blue_3}
            onPress={() => this.prev()}
          />
        )}
      </View>
    );
  };

  percentQuestion = () => {
    const last = this.state.index + 1;
    const total = this.state.total;
    const popupPercent = last * (95 / total);
    const progressPercent = last * (100 / total);
    // console.log(455, percent, popupPercent, progressPercent);
    return {popup: popupPercent, progress: progressPercent};
  };

  btnComplete = () => {
    const {user_answer} = this.props.submitData;
    const {children, isFinishQiz, navigation} = this.props;
    const numberAnswer = user_answer.length + '/' + children.length;
    if (isFinishQiz) {
      navigation.navigate('Congratulations', {numberAnswer: numberAnswer});
    } else {
      Alert.alert(
        'Thông báo',
        `Bạn đã trả lời ${numberAnswer} câu hỏi. Bạn muốn hoàn thành bài kiểm tra?`,
        [{text: 'Huỷ bỏ'}, {text: 'Đồng ý', onPress: () => this.acceptAnwer()}],
      );
    }
  };

  /**
   * Render the component
   */
  render = ({children, data, questions, keyPage, route} = this.props) => {
    // console.log(431, data);
    const {data: cur_data} = data;
    return (
      <View style={[styles.container, styles.fullScreen]}>
        <StatusBar hidden={true} />
        <View style={styles.viewHeader}>
          <TouchableOpacity
            style={styles.btnBack}
            onPress={() => route && route.goBack()}>
            <MaterialIcons name="keyboard-backspace" style={styles.iconBack} />
          </TouchableOpacity>
          <Text style={styles.txtTitle}>{questions[keyPage].titleSection}</Text>
          <View style={styles.btnTime}>
            <Image
              source={ic_clock}
              resizeMode="contain"
              style={styles.iconClock}
            />
            <CountDown
              minutes={cur_data.time_limit}
              endTime={() => this.confirmAnwer(1)}
            />
          </View>
          <TouchableOpacity style={styles.btnDone} onPress={this.btnComplete}>
            <Text style={styles.txtDone}>Hoàn thành</Text>
          </TouchableOpacity>
        </View>
        {this.renderScrollView(children)}
        <View style={styles.viewBottom}>
          <View style={styles.viewLeft}>
            <ImageBackground
              source={img_popup}
              resizeMode="contain"
              style={[
                styles.popup,
                {marginStart: this.percentQuestion().popup + '%'},
              ]}>
              {this.renderQuestions()}
            </ImageBackground>
            <View style={styles.viewProgress}>
              <View
                style={[
                  styles.progress,
                  {width: this.percentQuestion().progress + '%'},
                ]}
              />
            </View>
          </View>
          <View style={styles.viewRight}>
            {this.renderButtonPrev()}
            {this.renderButton()}
          </View>
        </View>
        {this.state.loading && (
          <ActivityIndicator
            size="large"
            color="orange"
            style={styles.loading}
          />
        )}
      </View>
    );
  };
}
function mapStateToProps(state) {
  return {
    data: state.reducersAccepted,
    isFinishQiz: state.reducerComponents.isFinish,
    submitData: state.submitData,
    submitAnswer: state.submitAnswer,
  };
}
export default connect(mapStateToProps)(OnboardingScreens);
